#################################################################################
#
#    mk-zprod - Automatically create Zope Python Products
#    Copyright (C) 2000 Erik Enge <erik@esol.no>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##################################################################################

__doc__     = "The python script makes a complete Zope Python Product."
__version__ = "2.0.0"


"""
This is how you describe your Python Product.  Notice that
you can have as many Classes, SubClasses and Attributes as
you feel like.

[<ProductName>,
 [image list],
 [<ClassName>, <classtype>, <classiconname>, 
  [[<attributename>, <attributetype>, <attributedefaultvalueorNone>]],
  [<SubClassName>, <subclasstype>, <subclassiconname>,
   [attribute list], 
   [subclass list]]]]
"""

import string

from zclass import ItemZClass
from zclass import FolderZClass
from zopeproduct import ZopeProduct


def add_class(class_list, print_prefix, product_name, product_directory, template_directory, dtml_directory):
	"""
	This function calls the necessary classes' methods to make
	the individual classes.  It accepts one list; [<ClassName>...]
	"""
	# We print with this prefix to make it look at bit nicer
	# to the user.  :)
	print_prefix = print_prefix + '\t'

	try: class_name = class_list[0]
	except:	class_name = None
	try: class_type = class_list[1]
	except:	class_type = None
	try: class_icon = class_list[2]
	except: class_icon = None
	try: class_attributes = class_list[3]
	except: class_attributes = None

	print print_prefix, class_name
	class_subclasses = []
	# For an explanation of this for-loop, read the next comments
	# 20 or so lines down in this file.
	for x in range(4, 100):
		try:
			class_subclasses.append(class_list[x][0])
		except:
			pass

	if string.lower(class_type) == 'item':
		zclass = ItemZClass(class_name,
							class_icon,
							class_attributes,
							class_subclasses,
							product_name,
							product_directory,
							template_directory,
							dtml_directory)
	elif string.lower(class_type) == 'folder':
		zclass = FolderZClass(class_name,
							  class_icon,
							  class_attributes,
							  class_subclasses,
							  product_name,
							  product_directory,
							  template_directory,
							  dtml_directory)

	# Call the methods that "makes" the class.
	try:
		zclass.add_class()
	except:
		print print_prefix + "Something ugly has occurred.  The zclass instance wasn't registered."
		print print_prefix + "Exiting."
		exit
	
	# If you have more than 96 classes on the same level, this for-loop
	# won't detect it.  To correct it, you could either up the 100 to
	# whatever you feel like, or maybe take another look at your class
	# structure.  I would suggest the latter.  :)
	for x in range(4, 100):
		# The reason we are starting at 4 is that the fourth item of
		# the list is the next class at same level.  For example the
		# Continent and Continent2 in the example list.
		try:
			# If we actually have onboard another class at same level,
			# we'll just call ourselves with this class' list as argument.
			# Neat, huh?  ;)
			add_class(class_list[x],
					  print_prefix,
					  product_name,
					  product_directory,
					  template_directory,
					  dtml_directory)
		except:
			pass

def copying():
	"Print copyright notice."
	print """
    mk-zprod version """+str(__version__)+""", Copyright (C) 2000  Erik Enge <erik@esol.no>
    mk-zprod comes with ABSOLUTELY NO WARRANTY; see the file COPYING.
    This is free software, and you are welcome to redistribute it under
    certain conditions; see the file COPYING.
	"""

def make_product(product_list):
	"""
	This function controls the process of adding classes to the
	Product and to make the __init__.py file.  This function also
	processes the [<ProductName>...] list and calls add_class() with
	the apropriate [<ClassName>...] list.

	It accepts one list; [<ProductName>...].
	"""
	copying()
	template_directory = 'tmpl/'
	image_directory = 'img/'
	dtml_directory = 'dtml/'
	
	product_name = product_list[0]
	product_images = product_list[1]
	product_classes = product_list[2]

	zproduct = ZopeProduct(product_name,
						   product_images,
						   template_directory,
						   image_directory)

	print 'Product name:', zproduct.name
	zproduct.add_product()

	print_prefix = ''

	add_class(product_classes,
			  print_prefix,
			  zproduct.name,
			  zproduct.name+'/',
			  template_directory,
			  dtml_directory)


list = ['ProductName',
		['icon.gif'],
		['ClassName', 'classmetatype', 'icon',
		 [['property', 'propertytype', 'propertydefaultvalueorNone']]]]

make_product(list)

# Here's an example product list.
#
#   list=['TravelAgent',
#	     ['Folder_icon.gif', 'dtmldoc.gif'],
#	     ['TravelAgent', 'folder', 'Folder_icon.gif',
#	      [['title', 'string', ''],
#          ['creator', 'int', '2']],
#	      ['Continent', 'folder', 'Folder_icon.gif',
#		   [['title', 'string', '']],
#		   ['Country', 'item', 'dtmldoc.gif',
#		    [['title', 'string', '']]]],
#		  ['Continent2', 'folder', 'Folder_icon.gif',
#		   [['title', 'string', '']]]]]



