#################################################################################
#
#    mk-zprod - Automatically create Zope Python Products
#    Copyright (C) 2000 Erik Enge <erik@esol.no>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##################################################################################

__doc__     = "Provides all classes and methods needed for a Zope Python Class."
__version__ = "2.0.1"

import string

from docstring import DocString

class ZClass(
	DocString):
	"This class contains all methods needed to create a Zope Python Class."
	
	def __init__(self, name, icon, attributes, subclasses, product_name, product_directory, template_directory, dtml_directory):
		"Initialize the class instance."
		self.name = name
		self.icon = icon
		self.attributes = attributes
		self.subclasses = subclasses
		self.lname = string.lower(self.name)
		self.product_directory = product_directory
		self.template_directory = template_directory
		self.dtml_directory = dtml_directory
		self.product_name = product_name

	def attriblist(self, type):
		"""
		Return the attribute list.  Possible types are:

		              1, just a plain list, sorted.
					  2, the list with default values, sorted.
					  3, the same as 1, but with id, unsorted.
		"""
		block = ""
		list1 = ""
		list2 = ""

		if type == 1:
			for attribute in self.attributes:
				if attribute[2] is None:
					list1 = list1 + attribute[0] + ", "
				else:
					list2 = list2 + attribute[0] + ", "
			return list1 + list2[:-2] # Strip the last comma.
		elif type == 2:
			list1 = ""
			list2 = ""
			for attribute in self.attributes:
				if attribute[2] is None:
					list1 = list1 + attribute[0] + ", "
				else:
					list2 = list2 + attribute[0] + "='" + str(attribute[2]) + "', "
			return ", " + list1 + list2[:-2] # Strip the last comma.
		elif type == 3:
			block = "id, "
			for attribute in self.attributes:
				block = block + attribute[0] + ", "
			return block

	def importblock(self):
		"The various imports at the top of a .py file."
		block = ""
		for subclass in self.subclasses:
			block = block + "import "+string.lower(subclass)+"\n"
		return block[:-1] # Strip the last newline character.

	def manage_addblock(self):
		"Return the manage_add block at the top of a Zope Python Class."
		block = open(self.template_directory+'manage_add.tmpl', 'r').read()
		return block % (self.name,
						self.lname,
						self.name,
						self.attriblist(3),
						self.name,
						self.name,
						self.attriblist(1))

	def properties(self):
		"Return the _properties tuple."
		block = "\n\t_properties = (\n"
		for attribute in self.attributes:
			block = block + "\t\t{'id':'"+ attribute[0]  +"', 'type':'"+ attribute[1] +"'},\n"
		return block + "\t\t)\n"

	def meta_types(self):
		"Return the meta_types tuple."
		if self.subclasses is not None:
			block = "\n\tmeta_types = (\n"
			for subclass in self.subclasses:
				block = block + "\t\t{'name':'"+ subclass +"', 'action':'manage_add"+ subclass +"Form'},\n"
			return block + "\t\t)\n"
		else:
			return ""

	def subadds(self):
		"Return the add methods of the subclasses."
		if self.subclasses is not None:
			block = "\n"
			for subclass in self.subclasses:
				block = block + "\tmanage_add"+ subclass +"Form = "+ string.lower(subclass) +".manage_add"+ subclass +"Form\n"
				block = block + "\tmanage_add"+ subclass +" = "+ string.lower(subclass) +".manage_add"+ subclass +"\n"
			return block
		else:
			return ""

	def setattriblist(self):
		"Return the self.x = x for every attribute x."
		if self.attributes is not None:
			block = "\n"
			for attribute in self.attributes:
				block = block + "\t\tself."+ attribute[0] +" = "+ attribute[0] +"\n"
			return block
		else:
			return ""

	def permissions(self):
		"Return an extra permission tuple for each subclass."
		if self.subclasses is not None:
			block = "\n"
			for subclass in self.subclasses:
				block = block + "\t\t('Add "+ subclass +"',\n"
				block = block + "\t\t ('manage_add"+ subclass +"', 'manage_add"+ subclass +"Form')),\n"
			return block[:-1] # Strip last newline character.
		else:
			return ""

	def additional_methods(self):
		"Returns additonal methods to the class."
		return ""

	def additional_tuples(self):
		"Returns additional tuples to the class."
		return ""

	def additional_htmlfiles(self):
		"Returns additonal methods to the class."
		return ""
		
	def class_block(self):
		"Return the class block."
		block = open(self.template_directory+'class.tmpl', 'r').read()
		return block % (self.name,
						'',
						self.name,
						self.product_name,
						self.icon,
						self.properties(),
						self.additional_tuples(),
						self.meta_types(),
						self.subadds(),
						self.permissions(),
						self.name,
						self.lname,
						self.additional_htmlfiles(),
						self.attriblist(2),
						self.setattriblist(),
						self.additional_methods())

	def compile_class(self):
		"The top of the .py file."
		block = open(self.template_directory+'top.tmpl', 'r').read()
		return block % (self.docheader(),
						self.importblock(),
						self.manage_addblock(),
						self.class_block())

	def write_class(self):
		"Write the ZClass instance to a file on disk."
		file = open(self.product_directory+self.lname+'.py', 'w')
		file.write(self.compile_class())
		file.close()

	def dtml_add(self):
		"Return an Add.dtml text."
		block = open(self.dtml_directory+'Add.dtml', 'r').read()
		attribmarkup = ""
		for attribute in self.attributes:
			block2 = open(self.dtml_directory+'Add_tr.dtml', 'r').read()
			attribmarkup = attribmarkup + block2 % (attribute[0],
													attribute[0],
													attribute[1])
		return block % (self.name,
						attribmarkup)

	def dtml_index(self):
		"Return an Index.dtml text."
		block = open(self.dtml_directory+'Index.dtml', 'r').read()
		return block

	def write_dtmlmethods(self):
		"Write Index and Add dtml files to disk."
		# Create the classnameAdd.dtml
		file = open(self.product_directory+self.lname+'Add.dtml', 'w')
		file.write(self.dtml_add())
		file.close
		# And then the classnameIndex.dtml
		file = open(self.product_directory+self.lname+'Index.dtml', 'w')
		file.write(self.dtml_index())
		file.close

	def add_class(self):
		"Setup a complete Zope Python Class."
		self.write_class()
		self.write_dtmlmethods()


class ItemZClass(
	ZClass):
	"Specific methods to the Item object type."

	def importblock(self):
		"The various imports at the top of a .py file."
		block = open(self.template_directory+'import.tmpl', 'r').read()
		block = block % ('from Globals import MessageDialog\n',
						 'import OFS.SimpleItem')
		return block
		
	def meta_types(self):
		"Return the meta_types tuple."
		return ""

	def additional_methods(self):
		"Return the manage_edit() method."
		block = open(self.template_directory+'manage_edit.tmpl', 'r').read()
		return block % (self.attriblist(1)+', ',
						self.setattriblist(),
						'%s',
						'% self.id')

	def additional_tuples(self):
		"Return the manage_option tuple."
		block = open(self.template_directory+'manage_options.tmpl', 'r').read()
		return block

	def additional_htmlfiles(self):
		"Return the manage_main HTMLFile string."
		return "\n\tmanage_main = HTMLFile('"+ self.lname +"Edit', globals())"

	def class_block(self):
		"Return the class block."
		block = open(self.template_directory+'class.tmpl', 'r').read()
		return block % (self.name,
						'\n\tOFS.SimpleItem.Item,',
						self.name,
						self.product_name,
						self.icon,
						self.properties(),
						self.additional_tuples(),
						self.meta_types(),
						self.subadds(),
						self.permissions(),
						self.name,
						self.lname,
						self.additional_htmlfiles(),
						self.attriblist(2),
						self.setattriblist(),
						self.additional_methods())

	def dtml_edit(self):
		"Return an Edit.dtml text."
		block = open(self.dtml_directory+'Edit.dtml', 'r').read()
		attribmarkup = ""
		for attribute in self.attributes:
			block2 = open(self.dtml_directory+'Edit_tr.dtml', 'r').read()
			attribmarkup = attribmarkup + block2 % (attribute[0],
													attribute[0],
													attribute[1],
													attribute[0])
		return block % (attribmarkup)

	def write_dtmlmethods(self):
		"Write Index and Add dtml files to disk."
		# Create the classnameAdd.dtml
		file = open(self.product_directory+self.lname+'Add.dtml', 'w')
		file.write(self.dtml_add())
		file.close
		# Then the classnameIndex.dtml
		file = open(self.product_directory+self.lname+'Index.dtml', 'w')
		file.write(self.dtml_index())
		file.close
		# And finally, the classnameEdit.dtml
		file = open(self.product_directory+self.lname+'Edit.dtml', 'w')
		file.write(self.dtml_edit())
		file.close


class FolderZClass(
	ZClass):
	"Specific methods to the Folder object type."

	def importblock(self):
		"The various imports at the top of a .py file."
		block = open(self.template_directory+'import.tmpl', 'r').read()
		block = block % ('',
						 'import OFS.Folder')
		for subclass in self.subclasses:
			block = block + "import "+string.lower(subclass)+"\n"
		return block[:-1] # Strip the last newline character.

	def class_block(self):
		"Return the class block."
		block = open(self.template_directory+'class.tmpl', 'r').read()
		return block % (self.name,
						'\n\tOFS.Folder.Folder,',
						self.name,
						self.product_name,
						self.icon,
						self.properties(),
						self.additional_tuples(),
						self.meta_types(),
						self.subadds(),
						self.permissions(),
						self.name,
						self.lname,
						self.additional_htmlfiles(),
						self.attriblist(2),
						self.setattriblist(),
						self.additional_methods())


