#################################################################################
#
#    mk-zprod - Automatically create Zope Python Products
#    Copyright (C) 2000 Erik Enge <erik@esol.no>
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
##################################################################################

__doc__ = """
          Provide classes and methods to make the framework for
		  a Zope Python Product; folders, __init__.py file.
		  """
__version__ = "2.0.1"

import os
import sys
import string

from docstring import DocString

class ZopeProduct(
	DocString):
	"""
	This class provides methods for creating the framework
	for a Zope Python Product.
	"""

	def __init__(self, name, images, template_directory, image_directory):
		"Initialize the class instance."
		self.name = name
		self.images = images
		self.product_directory = self.name
		self.template_directory = template_directory
		self.image_directory = image_directory
		self.lname = string.lower(self.name)

	def add_directory(self):
		"Add/make the necessary directories."
		delete_directories = 'n'
		delete_directories = raw_input('Do you want me to delete the current '+ self.name +' directory? (y/N) ')
		if delete_directories == ('y' or 'Y'):
			status = 'done.'
			print 'Deleting directories...',
			# Please be careful here.  If the self.name starts with a
			# / and you have root permissions...  Get it?  Ieesh.
			# Adding the code to safely do this will hurt as hell;
			# since the user can only create symlinks and make my life
			# suck.  On the other side.  If the user is doing this,
			# she probably wants to delete the whole file structure.
			try:
				# List all files in Product directory.
				for file in os.listdir(self.name):
					# For every file, delete it.
					os.remove(self.name+'/'+file)
				# Delete the now empty directory.
				os.rmdir(self.name)
			except:
				status = 'failed.'
				pass
			print status

		add_directories = 'n'
		add_directories = raw_input('Do you want me to create directory '+ self.name +'? (y/N) ')
		if add_directories == ('y' or 'Y'):
			status = 'done.'
			print 'Adding directories...',
			try:
				os.mkdir(self.name)
				os.mkdir(self.name+'/'+self.image_directory)
			except:
				status = 'failed.'
				pass
			print status

		try:
			os.stat(self.name)
		except OSError:
			print "Hm.  Somehow, the "+ self.name +"/ directory hasn't been created."
			print "Exiting."
			sys.exit()

	def imageblock(self):
		"Create the misc_ = {} block."
		block = ""
		for image in self.images:
			block = block + "\t'"+ image  +"': ImageFile('"+ self.image_directory + image +"', globals()),\n"
		return block[:-1] # Strip last newline character.

	def initfile(self):
		"Create the __init__.py file."
		block = open(self.template_directory+'__init__.tmpl', 'r').read()
		return block % (self.docheader(),
						self.imageblock(),
						self.lname,
						self.lname,
						self.name,
						self.lname,
						self.name,
						self.lname,
						self.name)

	def write_product(self):
		"Write the ZopeProduct instance to a file on disk."
		file = open(self.product_directory+'/__init__.py', 'w')
		file.write(self.initfile())
		file.close()

	def add_product(self):
		"Setup a complete product framework."
		self.add_directory()
		self.write_product()
			
			
